library sm2Cipher.test.impl_test;

import 'dart:convert' as convert;
import 'dart:typed_data';

import 'package:sm2cipher/sm2Cipher.dart';
import 'package:sm2cipher/src/sm2Cipher.dart';
import "package:test/test.dart";

String _formatBytesAsHexString(Uint8List bytes) {
  var result = StringBuffer();
  for (var i = 0; i < bytes.lengthInBytes; i++) {
    var part = bytes[i];
    result.write('${part < 16 ? '0' : ''}${part.toRadixString(16)}');
  }
  return result.toString();
}

void main() {
  group('Keys', () {
    test('Make request', () async {
      final convert.Utf8Encoder encoder = new convert.Utf8Encoder();
      final remote_pubkey =
          '02be8d8a7b5056de7a7074236100d094ebe86cce33d62469956203022af1f3e556';
      final my_kp = generateKeyPair();
      final data = 'abcdefg';
      final str_pri_key = strinifyPrivateKey(my_kp.privateKey);
      final str_pub_key = strinifyPublicKey(my_kp.publicKey);
      final enced = pubkeyEncryptRaw(str_pri_key, remote_pubkey,
          new Uint8List.fromList(encoder.convert(data)));
      final data_arr = <int>[];
      data_arr.addAll(encoder.convert(str_pub_key));
      data_arr.addAll(enced['enc'].toList());
      print(convert.base64.encode(data_arr));
      expect(true, true);
    });
    /*
    test('Test Verify Sign', () {
      final public_key =
          '022fc5ce94b28ce002450342e322dd50dba6229311ec11af1c94784859e6455fa3';
      final signature =
          'f1780ec798b8340b7e1a17dca8ec3b92f3f1a9b7e495b80e56854549eb3b61dd4fa7dd1d97f346c423fa9688317b71af95f8b0ac9741c9d23e8426675a7ddef0';
      final origin_text = 'test test';
      publicVerify(public_key, origin_text, signature);
    });
    */

    test('keep key', () {
      var bob = generateKeyPair();
      print(strinifyPrivateKey(bob.privateKey));
      print(strinifyPublicKey(bob.publicKey));
    });
    test('Encrypt and Decrypt', () {
      int micro_seconds = 0;
      for (var i = 0; i < 10; i++) {
        var aliceKeypair = generateKeyPair();
        final alic_pubkey = strinifyPublicKey(aliceKeypair.publicKey);
        final alic_prikey = strinifyPrivateKey(aliceKeypair.privateKey);

        var bob = generateKeyPair();
        var raw_str = '测试测试测试';
        final t1 = new DateTime.now().millisecondsSinceEpoch;
        var enc_map = pubkeyEncrypt(
            alic_prikey, strinifyPublicKey(bob.publicKey), raw_str);
        micro_seconds += (new DateTime.now().millisecondsSinceEpoch - t1);
        var enc_str = enc_map['enc'];
        var iv = enc_map['iv'];
        var decryptd = privateDecrypt(
            strinifyPrivateKey(bob.privateKey), alic_pubkey, enc_str);
        print('d:${decryptd}');
        expect(raw_str, equals(decryptd));
      }
      print('avg: ${micro_seconds / 100} ms');
    });
  });
}
